﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Media.Imaging;

namespace PerlinNoiseShader.Simplex2D
{
    /// <summary>
    /// consists of methods to calculate 2D simplex noise
    /// </summary>
    public class Simplex2DClass:NoiseMask
    {
        private ImageBrush oPImageBrush;
        private ImageBrush oGi1GradImageBrush;
        private ImageBrush oGi02GradImageBrush;
        private Simplex2DShaderEffect[] aSimplex2DShaderEffect;
        
        
        public Simplex2DClass():this((new Random()).Next())  {}
        
        public Simplex2DClass(int seed)
        {
            
            thisSeed=seed;
            oDrawRectangle = new Rectangle();
            iUIElement = oDrawRectangle;
            reInitialize(seed);
            aSimplex2DShaderEffect = new Simplex2DShaderEffect[4];
            for (int i = 0; i < 4; i++)
            {
                aSimplex2DShaderEffect[i] = new Simplex2DShaderEffect((Simplex2DShaderEffect.state)i);
            }
        }

        
        /// <summary>
        /// get the first gradient
        /// </summary>
        private void createGi1GradBrush()
        {
            int[][] grad3 = {new int[]{1,1,0},new int[]{-1,1,0},new int[]{1,-1,0},new int[]{-1,-1,0},
                    new int[]{1,0,1},new int[]{-1,0,1},new int[]{1,0,-1},new int[]{-1,0,-1},
                    new int[]{0,1,1},new int[]{0,-1,1},new int[]{0,1,-1},new int[]{0,-1,-1}};
            WriteableBitmap oWriteableBitmap = new WriteableBitmap(256, 1);
            for (int i = 0; i < 256; i++)
            {
                int perm = this.p[i] % 12;
                int[] vector = grad3[perm];
                int value = (255 << 24) | ((vector[0] + 1) << 16) | ((vector[1] + 1) << 8);
                oWriteableBitmap.Pixels[i] = value;
            }
            oWriteableBitmap.Invalidate();
            oGi1GradImageBrush = new ImageBrush();
            oGi1GradImageBrush.ImageSource = oWriteableBitmap;
        }


        /// <summary>
        /// get the second gradient
        /// </summary>
        private void createGi02GradBrush()
        {
            WriteableBitmap oWriteableBitmap = new WriteableBitmap(256, 256);
            Image oImage = new Image();
            oImage.Arrange(new Rect(0, 0, 256, 256));
            oImage.Measure(new Size(256, 256));
            oImage.Source = new WriteableBitmap(256, 256);
            Gi02GradShaderEffect oGi02GradShaderEffect = new Gi02GradShaderEffect();
            oGi02GradShaderEffect.pArray = oPImageBrush;
            oGi02GradShaderEffect.gi1GradArray = oGi1GradImageBrush;
            oImage.Effect = oGi02GradShaderEffect;
            oWriteableBitmap.Render(oImage, null);
            oWriteableBitmap.Invalidate();
            this.oGi02GradImageBrush = new ImageBrush();
            this.oGi02GradImageBrush.ImageSource = oWriteableBitmap;
        }

        
        
        public void reInitialize()
        {
            reInitialize((new Random()).Next());
        }

        public void reInitialize(int seed)
        {
            thisSeed = seed;
            oRandom = new Random(seed);
            oPImageBrush = getP();
            createGi1GradBrush();
            createGi02GradBrush();
        }

        /// <summary>
        /// calculates a single octave of 2D simplex noise
        /// </summary>
        /// <param name="freqXY">frequency at the x and y axis</param>
        /// <param name="offset">upper, left corner of the noise plane</param>
        /// <param name="amplitude"></param>
        /// <param name="absolute">noise or |noise|</param>
        /// <param name="c1">start color</param>
        /// <param name="c2">end color</param>
        /// <param name="oWriteableBitmap">destination bitmap</param>
        public void singleNoise(Size freqXY, Size offset, float amplitude, bool absolute, Color c1, Color c2, WriteableBitmap oWriteableBitmap)
        {
            Size testSize = new Size(oWriteableBitmap.PixelWidth, oWriteableBitmap.PixelHeight);
            if (!thisSize.Equals(testSize))
            {
                thisSize = testSize;
                if (oDrawRectangle != null) createRectangle();
            }
            Simplex2DShaderEffect oSimplex2DShaderEffect = absolute?aSimplex2DShaderEffect[1]:aSimplex2DShaderEffect[0];
            iUIElement.Effect = oSimplex2DShaderEffect;
            oSimplex2DShaderEffect.amplitude = absolute ? amplitude : 0.5f * amplitude;
            oSimplex2DShaderEffect.c1 = c1;
            oSimplex2DShaderEffect.c2 = c2;
            oSimplex2DShaderEffect.size = new Size(freqXY.Width * thisSize.Width, freqXY.Height * thisSize.Height);
            oSimplex2DShaderEffect.offset = new Size(offset.Width*freqXY.Width,offset.Height*freqXY.Height);
            oSimplex2DShaderEffect.pArray = oPImageBrush;
            oSimplex2DShaderEffect.gi1GradArray = oGi1GradImageBrush;
            oSimplex2DShaderEffect.gi02GradArray = oGi02GradImageBrush;
            oWriteableBitmap.Render(iUIElement, oCustomTransform);
            oWriteableBitmap.Invalidate();
        }

        /// <summary>
        /// calculates a single octave of 2D simplex noise to be mixed with other noise
        /// </summary>
        /// <param name="freqXY">frequency at the x and y axis</param>
        /// <param name="offset">upper, left corner of the noise plane</param>
        /// <param name="absolute">noise or |noise|</param>
        /// <param name="oWriteableBitmap">destination bitmap</param>
        public void noise(Size freqXY, Size offset, bool absolute, WriteableBitmap oWriteableBitmap)
        {
            Size testSize = new Size(oWriteableBitmap.PixelWidth, oWriteableBitmap.PixelHeight);
            if (!thisSize.Equals(testSize))
            {
                thisSize = testSize;
                if (oDrawRectangle != null) createRectangle();
            }
            Simplex2DShaderEffect oSimplex2DShaderEffect = absolute ? aSimplex2DShaderEffect[3] : aSimplex2DShaderEffect[2];
            iUIElement.Effect = oSimplex2DShaderEffect;
            oSimplex2DShaderEffect.size = new Size(freqXY.Width * thisSize.Width, freqXY.Height * thisSize.Height);
            oSimplex2DShaderEffect.offset = new Size(offset.Width * freqXY.Width, offset.Height * freqXY.Height);
            oSimplex2DShaderEffect.pArray = oPImageBrush;
            oSimplex2DShaderEffect.gi1GradArray = oGi1GradImageBrush;
            oSimplex2DShaderEffect.gi02GradArray = oGi02GradImageBrush;
            oWriteableBitmap.Render(iUIElement, oCustomTransform);
            oWriteableBitmap.Invalidate();
        }

    }

    
}
